/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/

#include <qapplication.h>
#include <qimage.h>
#include <qtextbrowser.h>
#include <qdragobject.h>
#include "pcplib.h"

extern char * defaultSerialDevice;

void PanCP::init()
{
    int r;
    const char **p;
    const QMimeSource *m;

    if (defaultSerialDevice)
	SerialDeviceComboBox->insertItem(defaultSerialDevice);
    SerialDeviceComboBox->insertItem(COMBO_SERIAL_DEVICE_0);
    SerialDeviceComboBox->insertItem(COMBO_SERIAL_DEVICE_1);
    SerialDeviceComboBox->setCurrentItem(0);
    
   for (p = serialSpeedVect; *p; p++)
       SerialSpeedComboBox->insertItem(*p);
   SerialSpeedComboBox->setCurrentItem(serialSpeedDefault);
    
    projOnPixmap = new QPixmap();
    m = QMimeSourceFactory::defaultFactory()->data( "proj-on.jpg" );
    QImageDrag::decode( m, *projOnPixmap );
    projOffPixmap = new QPixmap();
    m = QMimeSourceFactory::defaultFactory()->data( "proj-off.jpg" );
    QImageDrag::decode(m, *projOffPixmap );
    
    pcplib_init();

    r = query_power();
    
    if (r < 0)
	showErrorDialog();
    else  if (r > 0) /* Power is already on, go on with initialization */
	switchPower();    
}

void PanCP::switchPower()
{
    int p = pcp_get_power();
    int r;
    
    if (p == 1) { /* power is currently on, switch it off */
	if (pcp_powerOff_command() == PCP_OK) {
	    powerButton->setText(tr("Power On"));
	  Foto->setPixmap( *projOffPixmap ); 
	} else
	    showErrorDialog();
        return;
    } 
    /* power is off , switch it on */
     QApplication::setOverrideCursor( Qt::WaitCursor );
    r = pcp_powerOn_command();
    QApplication::restoreOverrideCursor();
    
    if (r == PCP_OK) {
	powerButton->setText(tr("Power Off"));
	Foto->setPixmap( *projOnPixmap ); 
	if (p == -1) /* first time ever? */
	    initializeControls();
    } else
	showErrorDialog();
}


void PanCP::initializeControls()
{
    int p;
 
    QApplication::setOverrideCursor( Qt::WaitCursor );
   
    query_projector_status();
        
    if ((p = pcp_get_lampOnTime()) == -1) {
	LampOnTimeLabel->setEnabled(false);
	LampOnTimeLCD->setEnabled(false);
    } else
	LampOnTimeLCD->display(p);
   
    if ((p=pcp_get_freeze())==-1)
	FreezeButton->setEnabled(false);
    else if (p == 1)
	FreezeButton->setText(tr("Freeze Off"));
    else
	FreezeButton->setText(tr("Freeze On"));
    
    if ((p=pcp_get_shutter())==-1)
	MuteButton->setEnabled(false);
    else if (p == 1)
	MuteButton->setText(tr("Mute Off"));
    else
	MuteButton->setText(tr("Mute On"));
    
    if ((p=pcp_get_lampPower())==-1)
	DimButton->setEnabled(false);
    else if (p==1)
	DimButton->setText(tr("Dim On"));
    else
	DimButton->setText(tr("Dim Off"));
       
    if ((p = pcp_get_brightness()) == -1) {
	BrightnessLabel->setEnabled(false);
       BrightnessSlider->setEnabled(false);
   } else
       BrightnessSlider->setValue(p);
   
   if ((p = pcp_get_color()) == -1) {
       ColorLabel->setEnabled(false);
       ColorSlider->setEnabled(false);
   } else
       ColorSlider->setValue(p);

   if ((p = pcp_get_contrast()) == -1) {
       ContrastLabel->setEnabled(false);
       ContrastSlider->setEnabled(false);
   }  else
       ContrastSlider->setValue(p);

   if ((p = pcp_get_sharpness()) == -1) {
       SharpnessLabel->setEnabled(false);
       SharpnessSlider->setEnabled(false);
   }  else
       SharpnessSlider->setValue(p);
   
   if ((p = pcp_get_tint()) == -1) {
       TintLabel->setEnabled(false);
       TintSlider->setEnabled(false);
   } else
       TintSlider->setValue(p);

    
   if ((p = pcp_get_backgroundColor()) != -1) {
       BackButtonGroup->setEnabled(true);
       if (p == 1)
	   BlackRadioButton->setChecked(true);
       else
	   BlueRadioButton->setChecked(true);
   }
   
   if ((p = pcp_get_pictureMode()) != -1) { 
       PictureButtonGroup->setEnabled(true);
       if (p == 1)
	   DynRadioButton->setChecked(true);
       else
	   NatRadioButton->setChecked(true);
   }
   
    PowerButtonGroup->setEnabled(true);
    SliderButtonGroup->setEnabled(true);
    
    QApplication::restoreOverrideCursor();
}


void PanCP::showErrorDialog()
{
    ErrorDialog * dial = new ErrorDialog();
    dial->exec();
    delete dial;
}


void PanCP::switchFreeze()
{
    QString tmp;
    int p = pcp_get_freeze();
    if (p == -1) return;
    if (p == 1)
	tmp = tr("Freeze On");
    else
	tmp = tr("Freeze Off");
    p = 1-p;
    if (pcp_freeze_command(p) == PCP_OK)
	    FreezeButton->setText(tmp);
	else
	    showErrorDialog();
}

void PanCP::switchLamp()
{
    QString tmp;
    int p = pcp_get_lampPower();
    if (p == -1) return;
    if (p == 1)
	tmp = tr("Dim Off");
    else
	tmp = tr("Dim On");
    p = 1-p;
    if (pcp_lampPower_command(p) == PCP_OK)
	    DimButton->setText(tmp);
	else
	    showErrorDialog();
}

void PanCP::switchMute()
{
    QString tmp;
    int p = pcp_get_shutter();
    if (p == -1) return;
    if (p == 1)
	tmp = tr("Mute On");
    else
	tmp = tr("Mute Off");
    if (pcp_shutter_command() == PCP_OK)
	    MuteButton->setText(tmp);
    else
	    showErrorDialog();
}



void PanCP::changeSerialDevice()
{
    int p;
    QString tmp;
    char log[128];
    
    tmp = SerialDeviceComboBox->text(SerialDeviceComboBox->currentItem());
    p = pcp_set_serialDevice(tmp, log);
    if (p) {
	QString err(tr("ERROR: ")+tmp);
	ErrorDialog *d = new ErrorDialog();
	d->TextLabel->setText(err);
	d->TextBrowser->setText(log);
	d->exec();
	delete d;
	SerialDeviceComboBox->setCurrentItem(0);
	err = SerialDeviceComboBox->currentText();
	pcp_set_serialDevice(err, NULL);
    }
}

void PanCP::changeSerialSpeed()
{
    int p;
    int idx;
    char log[128];
    
    idx= SerialSpeedComboBox->currentItem();
    p = pcp_set_serialSpeed(idx, log);
    if (p) {
	QString err(tr("ERROR:")+log);
	ErrorDialog *d = new ErrorDialog();
	d->TextLabel->setText(err);
	d->exec();
	delete d;
	SerialSpeedComboBox->setCurrentItem(serialSpeedDefault);
	pcp_set_serialSpeed(serialSpeedDefault, NULL);
    }
}

void PanCP::changeTimeout()
{
    int  v = TimeoutSpinBox->value();
    pcp_set_timeoutDefault(v);
}


void PanCP::showAboutDialog()
{
    AboutDialog * d = new AboutDialog();
    d->exec();
    delete d;
}


void PanCP::changeContrast()
{
   int v = ContrastSlider->value();
   pcp_contrast_command(v);
}

void PanCP::changeBrightness()
{
    int v = BrightnessSlider->value();
    pcp_brightness_command(v);
}

void PanCP::changeTint()
{
    int v = TintSlider->value();
    pcp_tint_command(v);
}

void PanCP::changeColor()
{
    int v = ColorSlider->value();
    pcp_color_command(v);
}

void PanCP::changeSharpness()
{
    int v = SharpnessSlider->value();
    pcp_sharpness_command(v);
}

void PanCP::changeBackground()
{
    int v = BlackRadioButton->isChecked();
    pcp_backgroundColor_command(v);
}

void PanCP::changePictureMode()
{
    int v = DynRadioButton->isChecked();
    pcp_pictureMode_command(v);
}


